@@ -13,7 +13,7 @@ use crate::{
     error::{ErrorFormatter, PrettyError},
     hub::{Global, GlobalIdentityHandlerFactory, HalApi, Storage, Token},
     id,
-    memory_init_tracker::{MemoryInitKind, MemoryInitTrackerAction},
+    init_tracker::MemoryInitKind,
     pipeline::PipelineFlags,
     resource::{Texture, TextureView},
     track::{StatefulTrackerSubset, TextureSelector, UsageConflict},
@@ -55,6 +55,7 @@ pub enum LoadOp {
 #[derive(Copy, Clone, Debug, Hash, Eq, PartialEq)]
 #[cfg_attr(any(feature = "serial-pass", feature = "trace"), derive(Serialize))]
 #[cfg_attr(any(feature = "serial-pass", feature = "replay"), derive(Deserialize))]
+#[cfg_attr(feature = "serde", serde(rename_all = "kebab-case"))]
 pub enum StoreOp {
     /// Discards the content of the render target. If you don't care about the contents of the target, this can be faster.
     Discard = 0,
@@ -222,8 +223,8 @@ enum OptionalState {
 
 impl OptionalState {
     fn require(&mut self, require: bool) {
-        if require && *self == OptionalState::Unused {
-            *self = OptionalState::Required;
+        if require && *self == Self::Unused {
+            *self = Self::Required;
         }
     }
 }
@@ -268,7 +269,7 @@ struct VertexBufferState {
 }
 
 impl VertexBufferState {
-    const EMPTY: Self = VertexBufferState {
+    const EMPTY: Self = Self {
         total_size: 0,
         stride: 0,
         rate: VertexStepMode::Vertex,
@@ -411,6 +412,8 @@ pub enum RenderPassErrorInner {
     InvalidResolveSourceSampleCount,
     #[error("resolve target must have a sample count of 1")]
     InvalidResolveTargetSampleCount,
+    #[error("surface texture is dropped before the render pass is finished")]
+    SurfaceTextureDropped,
     #[error("not enough memory left")]
     OutOfMemory,
     #[error("unable to clear non-present/read-only depth")]
@@ -730,6 +733,9 @@ impl<'a, A: HalApi> RenderPassInfo<'a, A> {
         }
 
         for ra in self.render_attachments {
+            if !texture_guard.contains(ra.texture_id.value.0) {
+                return Err(RenderPassErrorInner::SurfaceTextureDropped);
+            }
             let texture = &texture_guard[ra.texture_id.value];
             check_texture_usage(texture.desc.usage, TextureUsages::RENDER_ATTACHMENT)?;
 
@@ -876,7 +882,7 @@ impl<G: GlobalIdentityHandlerFactory> Global<G> {
                             .map_err(|_| RenderCommandError::InvalidBindGroup(bind_group_id))
                             .map_pass_err(scope)?;
                         bind_group
-                            .validate_dynamic_bindings(&temp_offsets)
+                            .validate_dynamic_bindings(&temp_offsets, &cmd_buf.limits)
                             .map_pass_err(scope)?;
 
                         // merge the resource tracker in
@@ -889,14 +895,7 @@ impl<G: GlobalIdentityHandlerFactory> Global<G> {
                         cmd_buf.buffer_memory_init_actions.extend(
                             bind_group.used_buffer_ranges.iter().filter_map(|action| {
                                 match buffer_guard.get(action.id) {
-                                    Ok(buffer) => buffer
-                                        .initialization_status
-                                        .check(action.range.clone())
-                                        .map(|range| MemoryInitTrackerAction {
-                                            id: action.id,
-                                            range,
-                                            kind: action.kind,
-                                        }),
+                                    Ok(buffer) => buffer.initialization_status.check_action(action),
                                     Err(_) => None,
                                 }
                             }),
@@ -1067,14 +1066,11 @@ impl<G: GlobalIdentityHandlerFactory> Global<G> {
                         state.index.update_limit();
 
                         cmd_buf.buffer_memory_init_actions.extend(
-                            buffer
-                                .initialization_status
-                                .check(offset..end)
-                                .map(|range| MemoryInitTrackerAction {
-                                    id: buffer_id,
-                                    range,
-                                    kind: MemoryInitKind::NeedsInitializedMemory,
-                                }),
+                            buffer.initialization_status.create_action(
+                                buffer_id,
+                                offset..end,
+                                MemoryInitKind::NeedsInitializedMemory,
+                            ),
                         );
 
                         let bb = hal::BufferBinding {
@@ -1122,14 +1118,11 @@ impl<G: GlobalIdentityHandlerFactory> Global<G> {
                         vertex_state.bound = true;
 
                         cmd_buf.buffer_memory_init_actions.extend(
-                            buffer
-                                .initialization_status
-                                .check(offset..(offset + vertex_state.total_size))
-                                .map(|range| MemoryInitTrackerAction {
-                                    id: buffer_id,
-                                    range,
-                                    kind: MemoryInitKind::NeedsInitializedMemory,
-                                }),
+                            buffer.initialization_status.create_action(
+                                buffer_id,
+                                offset..(offset + vertex_state.total_size),
+                                MemoryInitKind::NeedsInitializedMemory,
+                            ),
                         );
 
                         let bb = hal::BufferBinding {
@@ -1382,14 +1375,11 @@ impl<G: GlobalIdentityHandlerFactory> Global<G> {
                         }
 
                         cmd_buf.buffer_memory_init_actions.extend(
-                            indirect_buffer
-                                .initialization_status
-                                .check(offset..end_offset)
-                                .map(|range| MemoryInitTrackerAction {
-                                    id: buffer_id,
-                                    range,
-                                    kind: MemoryInitKind::NeedsInitializedMemory,
-                                }),
+                            indirect_buffer.initialization_status.create_action(
+                                buffer_id,
+                                offset..end_offset,
+                                MemoryInitKind::NeedsInitializedMemory,
+                            ),
                         );
 
                         match indexed {
@@ -1472,14 +1462,11 @@ impl<G: GlobalIdentityHandlerFactory> Global<G> {
                             .map_pass_err(scope);
                         }
                         cmd_buf.buffer_memory_init_actions.extend(
-                            indirect_buffer
-                                .initialization_status
-                                .check(offset..end_offset)
-                                .map(|range| MemoryInitTrackerAction {
-                                    id: buffer_id,
-                                    range,
-                                    kind: MemoryInitKind::NeedsInitializedMemory,
-                                }),
+                            indirect_buffer.initialization_status.create_action(
+                                buffer_id,
+                                offset..end_offset,
+                                MemoryInitKind::NeedsInitializedMemory,
+                            ),
                         );
 
                         let begin_count_offset = count_buffer_offset;
@@ -1493,14 +1480,11 @@ impl<G: GlobalIdentityHandlerFactory> Global<G> {
                             .map_pass_err(scope);
                         }
                         cmd_buf.buffer_memory_init_actions.extend(
-                            count_buffer
-                                .initialization_status
-                                .check(count_buffer_offset..end_count_offset)
-                                .map(|range| MemoryInitTrackerAction {
-                                    id: count_buffer_id,
-                                    range,
-                                    kind: MemoryInitKind::NeedsInitializedMemory,
-                                }),
+                            count_buffer.initialization_status.create_action(
+                                count_buffer_id,
+                                count_buffer_offset..end_count_offset,
+                                MemoryInitKind::NeedsInitializedMemory,
+                            ),
                         );
 
                         match indexed {
@@ -1642,14 +1626,7 @@ impl<G: GlobalIdentityHandlerFactory> Global<G> {
                                 .buffer_memory_init_actions
                                 .iter()
                                 .filter_map(|action| match buffer_guard.get(action.id) {
-                                    Ok(buffer) => buffer
-                                        .initialization_status
-                                        .check(action.range.clone())
-                                        .map(|range| MemoryInitTrackerAction {
-                                            id: action.id,
-                                            range,
-                                            kind: action.kind,
-                                        }),
+                                    Ok(buffer) => buffer.initialization_status.check_action(action),
                                     Err(_) => None,
                                 }),
                         );
